@@ -10,7 +10,45 @@ wrl_header = """#VRML V2.0 utf8
 """
 
 
-def get_3Dmodel(
+def get_StepModel(component_uuid, footprint_info, kicad_mod):
+    logging.info(f"Downloading STEP Model ...")
+
+    # `qAxj6KHrDKw4blvCG8QJPs7Y` is a constant in
+    # https://modules.lceda.cn/smt-gl-engine/0.8.22.6032922c/smt-gl-engine.js
+    # and points to the bucket containing the step files.
+
+    response = requests.get(
+        f"https://modules.easyeda.com/qAxj6KHrDKw4blvCG8QJPs7Y/{component_uuid}"
+    )
+    if response.status_code == requests.codes.ok:
+        ensure_footprint_lib_directories_exist(footprint_info)
+        filename = f"{footprint_info.output_dir}/{footprint_info.footprint_lib}/{footprint_info.model_dir}/{footprint_info.footprint_name}.step"
+        with open(filename, "wb") as f:
+            f.write(response.content)
+
+        logging.info(f"STEP model created at {filename}")
+
+        if footprint_info.model_base_variable:
+            if footprint_info.model_base_variable.startswith("$"):
+                path_name = f'"{footprint_info.model_base_variable}/{footprint_info.footprint_name}.step"'
+            else:
+                path_name = f'"$({footprint_info.model_base_variable})/{footprint_info.footprint_name}.step"'
+        else:
+            path_name = filename
+
+        kicad_mod.append(
+            Model(
+                filename=path_name,
+            )
+        )
+        logging.info(f"added {path_name} to footprint")
+
+    else:
+        logging.error("request error, no Step model found")
+        return
+
+
+def get_WrlModel(
     component_uuid,
     footprint_info,
     kicad_mod,
@@ -19,7 +57,7 @@ def get_3Dmodel(
     translationZ,
     rotation,
 ):
-    logging.info("creating 3D model ...")
+    logging.info("Creating WRL model ...")
 
     response = requests.get(
         f"https://easyeda.com/analyzer/api/3dmodel/{component_uuid}"
@@ -150,26 +188,23 @@ Shape{{
     x_middle = (max(x_list) + min(x_list)) / 2
     y_middle = (max(y_list) + min(y_list)) / 2
 
-    if not os.path.exists(
-        f"{footprint_info.output_dir}/{footprint_info.footprint_lib}"
-    ):
-        os.makedirs(f"{footprint_info.output_dir}/{footprint_info.footprint_lib}")
-    if not os.path.exists(
-        f"{footprint_info.output_dir}/{footprint_info.footprint_lib}/packages3d"
-    ):
-        os.makedirs(
-            f"{footprint_info.output_dir}/{footprint_info.footprint_lib}/packages3d"
-        )
+    ensure_footprint_lib_directories_exist(footprint_info)
 
-    filename = f"{footprint_info.output_dir}/{footprint_info.footprint_lib}/packages3d/{footprint_info.footprint_name}.wrl"
+    filename = f"{footprint_info.output_dir}/{footprint_info.footprint_lib}/{footprint_info.model_dir}/{footprint_info.footprint_name}.wrl"
     with open(filename, "w") as f:
         f.write(wrl_content)
 
     if footprint_info.model_base_variable:
-        path_name = f'"$({footprint_info.model_base_variable})/{footprint_info.footprint_name}.wrl"'
+        if footprint_info.model_base_variable.startswith("$"):
+            path_name = f'"{footprint_info.model_base_variable}/{footprint_info.footprint_name}.wrl"'
+        else:
+            path_name = f'"$({footprint_info.model_base_variable})/{footprint_info.footprint_name}.wrl"'
     else:
         dirname = os.getcwd().replace("\\", "/").replace("/footprint", "")
-        path_name = f"{dirname}/{filename}"
+        if os.path.isabs(filename):
+            path_name = filename
+        else:
+            path_name = f"{dirname}/{filename}"
 
         # Calculate the translation between the center of the pads and the center of the whole footprint
     translation_x = translationX - footprint_info.origin[0]
@@ -180,11 +215,32 @@ Shape{{
     translation_y_inches = -translation_y / 100 - y_middle / 10
     translation_z_inches = translation_z / 25.4
 
-    kicad_mod.append(
-        Model(
-            filename=path_name,
-            at=[translation_x_inches, translation_y_inches, translation_z_inches],
-            rotate=[-float(axis_rotation) for axis_rotation in rotation.split(",")],
+    # Check if a model has already been added to the footprint to prevent duplicates
+    if any(isinstance(child, Model) for child in kicad_mod.getAllChilds()):
+        logging.info(f"WRL model created at {filename}")
+        logging.info(
+            f"WRL model was not added to the footprint to prevent duplicates with STEP model"
+        )
+    else:
+        kicad_mod.append(
+            Model(
+                filename=path_name,
+                at=[translation_x_inches, translation_y_inches, translation_z_inches],
+                rotate=[-float(axis_rotation) for axis_rotation in rotation.split(",")],
+            )
+        )
+        logging.info(f"added {path_name} to footprintc")
+
+
+def ensure_footprint_lib_directories_exist(footprint_info):
+    if not os.path.exists(
+        f"{footprint_info.output_dir}/{footprint_info.footprint_lib}"
+    ):
+        os.makedirs(f"{footprint_info.output_dir}/{footprint_info.footprint_lib}")
+
+    if not os.path.exists(
+        f"{footprint_info.output_dir}/{footprint_info.footprint_lib}/{footprint_info.model_dir}"
+    ):
+        os.makedirs(
+            f"{footprint_info.output_dir}/{footprint_info.footprint_lib}/{footprint_info.model_dir}"
         )
-    )
-    logging.info(f"added {path_name} to footprint")
